/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  drv3946-q1-RegMap.h - Declaration file for utility functions and global variables
 *  DRV3946-Q1EVM_FW
 *  10/14/2021
 *
 ******************************************************************************/
#ifndef DRV3946_REGMAP_H_
#define DRV3946_REGMAP_H_

#include "../dataTypeDefinition.h"

/*******************************************************************************
 * MACROs
 *******************************************************************************/

/*******************************************************************************
 * DRV3946 SPI Register ADDRESS
 *******************************************************************************/
#define SPI_REG_STATUS0        (uint16_t)(0x01)       // STATUS0 Register
#define SPI_REG_STATUS1        (uint16_t)(0x02)       // STATUS1 Register
#define SPI_REG_STATUS2        (uint16_t)(0x03)       // STATUS2 Register
#define SPI_REG_STATUS3        (uint16_t)(0x04)       // STATUS3 Register
#define SPI_REG_STATUS4        (uint16_t)(0x0A)       // STATUS4 Register
#define SPI_REG_STATUS5        (uint16_t)(0x0B)       // STATUS5 Register
#define SPI_REG_MEAS0          (uint16_t)(0x05)       // MEAS1 Register
#define SPI_REG_MEAS1          (uint16_t)(0x06)       // MEAS2 Register
#define SPI_REG_MEAS2          (uint16_t)(0x07)       // MEAS3 Register
#define SPI_REG_MEAS3          (uint16_t)(0x08)       // MEAS4 Register
#define SPI_REG_MEAS4          (uint16_t)(0x09)       // MEAS5 Register
#define SPI_REG_MEAS5          (uint16_t)(0x0C)       // MEAS6 Register     --> 2p0 addition
#define SPI_REG_MEAS6          (uint16_t)(0x0D)       // MEAS7 Register     --> 2p0 addition
#define SPI_REG_CONFIGA0       (uint16_t)(0x10)       // CONFIGA0 Register
#define SPI_REG_CONFIGA1       (uint16_t)(0x11)       // CONFIGA1 Register
#define SPI_REG_CONFIGA2       (uint16_t)(0x12)       // CONFIGA2 Register
#define SPI_REG_CONFIGA3       (uint16_t)(0x13)       // CONFIGA3 Register
#define SPI_REG_CONFIGA4       (uint16_t)(0x14)       // CONFIGA4 Register
#define SPI_REG_CONFIGA5       (uint16_t)(0x15)       // CONFIGA5 Register
#define SPI_REG_CONFIGA6       (uint16_t)(0x16)       // CONFIGA6 Register
#define SPI_REG_CONFIGB0       (uint16_t)(0x17)       // CONFIGB0 Register
#define SPI_REG_CONFIGB1       (uint16_t)(0x18)       // CONFIGB1 Register
#define SPI_REG_CONFIGB2       (uint16_t)(0x19)       // CONFIGB2 Register      --> address change for 2p0
#define SPI_REG_CONFIGB3       (uint16_t)(0x1A)       // CONFIGB3 Register      --> 2p0 addition
#define SPI_REG_CONFIGB4       (uint16_t)(0x1B)       // CONFIGB4 Register      --> 2p0 addition
#define SPI_REG_CMD0           (uint16_t)(0x1C)       // CMD_CONFIG Register
#define SPI_REG_CMD1           (uint16_t)(0x1D)       // CRC_PROTECTED_CMD Register
#define SPI_REG_CMD2           (uint16_t)(0x1E)       // CMD_CRC Register

/*******************************************************************************
 * Structure
 *******************************************************************************/

//  Structure variables for the DRV3946-Q1 registers.
typedef struct DRV3946_REG
{
    // starting address = 0x09C0, Number of bytes = 50 bytes
    uint16_t dummy_filler;
    uint16_t status0_reg;
    uint16_t status1_reg;
    uint16_t status2_reg;
    uint16_t status3_reg;
    uint16_t status4_reg;
    uint16_t status5_reg;
    uint16_t meas0_reg;
    uint16_t meas1_reg;
    uint16_t meas2_reg;
    uint16_t meas3_reg;
    uint16_t meas4_reg;
    uint16_t meas5_reg;             //2p0 addition
    uint16_t meas6_reg;             //2p0 addition
    uint16_t configA0_reg;
    uint16_t configA1_reg;
    uint16_t configA2_reg;
    uint16_t configA3_reg;
    uint16_t configA4_reg;
    uint16_t configA5_reg;
    uint16_t configA6_reg;
    uint16_t configB0_reg;
    uint16_t configB1_reg;
    uint16_t configB2_reg;
    uint16_t configB3_reg;           //2p0 addition
    uint16_t configB4_reg;           //2p0 addition
    uint16_t cmd0_reg;
    uint16_t cmd1_reg;
    uint16_t cmd2_reg;
} DRV3946_Q1_REG_t;

#endif // DRV3946_REGMAP_H_
