/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/********************************************************************************
 *
 *  drv3946-q1.c - Definition file for utility functions
 *  DRV3946-Q1EVM_FIRMWARE
 *  10/14/2021  --> 7/20/2023
 *  Murugavel Raju --> Ibinu A N
 *
 *******************************************************************************/
#define FIRMWARE_VERSION 1.00

#include "drv3946-q1/drv3946-q1.h"
#include "crc.h"
#include "spi.h"
#include "uart.h"
#include "Serial_Cmd_Monitor.h"

/********************************************************************************
 * Data Structures exported for GUI Composer
 *******************************************************************************/
#pragma DATA_SECTION(gBrushedObj,".drvVarSection");
#pragma DATA_SECTION(gDeviceSpiReg,".drvRegSection");
volatile DRV3946_Q1_REG_t gDeviceSpiReg;
volatile DRV3946_Q1_Device_Obj_t gBrushedObj;

/*******************************************************************************
 *  Variable definitions
 *******************************************************************************/
volatile uint8_t configAx[14] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0}; // Initialize array for CONFIGAX upper and lower bytes
volatile uint8_t configBx[10] = {0,0,0,0,0,0,0,0,0,0};                  // Initialize array for CONFIGBX upper and lower bytes

/*******************************************************************************
 * API IMPLEMENTATION
 *******************************************************************************/
// gui_deviceControlUpdateTask - Synchronize values in the global data structure written to by
// the GUI with the hardware.
void gui_deviceControlUpdateTask()
{
    // read_pin macros return raw byte port value and convert to binary flags
    uint8_t drvoffState1A = read_ENABLE1_A_pin;
    uint8_t drvoffState2A = read_ENABLE2_A_pin;
    uint8_t drvoffState1B = read_ENABLE1_B_pin;
    uint8_t drvoffState2B = read_ENABLE2_B_pin;

    if (drvoffState1A > 0)
        drvoffState1A = 1;

    if (drvoffState2A > 0)
        drvoffState2A = 1;

    if (drvoffState1B > 0)
        drvoffState1B = 1;

    if (drvoffState2B > 0)
        drvoffState2B = 1;
    // end read_pin macros

    if (!gBrushedObj.drv_SPI_mode)
    {
        if (gBrushedObj.DRVA_sel && !(read_SW_EN2_pin))        // GUI slider set for Driver A
        {
            driver_A_power_on();                               // Driver A power on
            drv_RESET();                                       // Clear fault and nad_init after power on
        }

        if (gBrushedObj.DRVB_sel && !(read_SW_EN1_pin))        // GUI slider set for Driver B
        {
            driver_B_power_on();                               // Driver B power on
            drv_RESET();                                       // Clear fault and nad_init after power on
        }
    }
    else if (gBrushedObj.drv_SPI_mode)
    {
        if (read_SW_EN1_pin || read_SW_EN2_pin)
        {
            set_SW_EN1_lo;                                     // Driver A power 0n
            set_SW_EN2_lo;                                     // Driver B power 0n
            P4OUT |= DRV_A_LED | DRV_B_LED;                    // Driver A LED on and Driver B LED on
            __delay_cycles(8000);                              // Power on settling delay 0.5 ms
            drv_RESET();                                       // Clear fault and nad_init after power on for all devices
        }
    }

    // sync EN1 and EN2 in data structure with actual pin state
    if (gBrushedObj.DRVA_enable1 != drvoffState1A)
    {
        if (gBrushedObj.DRVA_enable1)
        {
            set_ENABLE1_A_hi;                              // Enable1A high
        }
        else
        {
            set_ENABLE1_A_lo;                              // Enable1A low
        }
    }

    if (gBrushedObj.DRVA_enable2 != drvoffState2A)
    {
        if (gBrushedObj.DRVA_enable2)
        {
            set_ENABLE2_A_hi;                              // Enable2A high
        }
        else
        {
            set_ENABLE2_A_lo;                              // Enable2A low
        }
    }

    if (gBrushedObj.DRVB_enable1 != drvoffState1B)
    {
        if (gBrushedObj.DRVB_enable1)
        {
            set_ENABLE1_B_hi;                              // Enable1B high
        }
        else
        {
            set_ENABLE1_B_lo;                              // Enable1B low
        }
    }

    if (gBrushedObj.DRVB_enable2 != drvoffState2B)
    {
        if (gBrushedObj.DRVB_enable2)
        {
            set_ENABLE2_B_hi;                              // Enable2B high
        }
        else
        {
            set_ENABLE2_B_lo;                              // Enable2B low
        }
    }

    // Clear fault function
    if (gBrushedObj.clrFlt)
    {
        spi_writeRegister(SPI_REG_CMD1, ((uint16_t) spi_readRegister(SPI_REG_CMD1) | 0x8000)); // write CLR_FAULT bit high
        __delay_cycles(100);                               // 6.25 us settling delay
        gBrushedObj.clrFlt = 0;                            // reset widget value
    }// End Clear fault function

    // Fault LEDs update
    if (read_FAULT_A_pin)
    {
        P2OUT &= ~FAULT_A_LED;                             // FAULT_A LED off
        gBrushedObj.DRVA_FAULT = 1;                        // GUI fault indicator
    }
    else if (!read_FAULT_A_pin)
    {
        P2OUT |= FAULT_A_LED;                              // FAULT_A LED on
        gBrushedObj.DRVA_FAULT = 0;                        // GUI fault indicator
    }

    if (read_FAULT_B_pin)
    {
        P2OUT &= ~FAULT_B_LED;                             // FAULT_B LED off
        gBrushedObj.DRVB_FAULT = 1;                        // GUI fault indicator

    }
    else if (!read_FAULT_B_pin)
    {
        P2OUT |= FAULT_B_LED;                              // FAULT_B LED on
        gBrushedObj.DRVB_FAULT = 0;                        // GUI fault indicator
    }
    // End fault LEDs update

} // end gui_deviceControlUpdateTask()

void gui_regRead(void)
{
    gDeviceSpiReg.status0_reg = (uint16_t) spi_readRegister(SPI_REG_STATUS0);
    gDeviceSpiReg.status1_reg = (uint16_t) spi_readRegister(SPI_REG_STATUS1);
    gDeviceSpiReg.status2_reg = (uint16_t) spi_readRegister(SPI_REG_STATUS2);
    gDeviceSpiReg.status3_reg = (uint16_t) spi_readRegister(SPI_REG_STATUS3);
    gDeviceSpiReg.status4_reg = (uint16_t) spi_readRegister(SPI_REG_STATUS4);
    gDeviceSpiReg.status5_reg = (uint16_t) spi_readRegister(SPI_REG_STATUS5);
    gDeviceSpiReg.meas0_reg = (uint16_t) spi_readRegister(SPI_REG_MEAS0);
    gDeviceSpiReg.meas1_reg = (uint16_t) spi_readRegister(SPI_REG_MEAS1);
    gDeviceSpiReg.meas2_reg = (uint16_t) spi_readRegister(SPI_REG_MEAS2);
    gDeviceSpiReg.meas3_reg = (uint16_t) spi_readRegister(SPI_REG_MEAS3);
    gDeviceSpiReg.meas4_reg = (uint16_t) spi_readRegister(SPI_REG_MEAS4);
    gDeviceSpiReg.meas5_reg = (uint16_t) spi_readRegister(SPI_REG_MEAS5);           //2p0 addition
    gDeviceSpiReg.meas6_reg = (uint16_t) spi_readRegister(SPI_REG_MEAS6);           //2p0 addition
    gDeviceSpiReg.configA0_reg = (uint16_t) spi_readRegister(SPI_REG_CONFIGA0);
    gDeviceSpiReg.configA1_reg = (uint16_t) spi_readRegister(SPI_REG_CONFIGA1);
    gDeviceSpiReg.configA2_reg = (uint16_t) spi_readRegister(SPI_REG_CONFIGA2);
    gDeviceSpiReg.configA3_reg = (uint16_t) spi_readRegister(SPI_REG_CONFIGA3);
    gDeviceSpiReg.configA4_reg = (uint16_t) spi_readRegister(SPI_REG_CONFIGA4);
    gDeviceSpiReg.configA5_reg = (uint16_t) spi_readRegister(SPI_REG_CONFIGA5);
    gDeviceSpiReg.configA6_reg = (uint16_t) spi_readRegister(SPI_REG_CONFIGA6);
    gDeviceSpiReg.configB0_reg = (uint16_t) spi_readRegister(SPI_REG_CONFIGB0);
    gDeviceSpiReg.configB1_reg = (uint16_t) spi_readRegister(SPI_REG_CONFIGB1);
    gDeviceSpiReg.configB2_reg = (uint16_t) spi_readRegister(SPI_REG_CONFIGB2);
    gDeviceSpiReg.configB3_reg = (uint16_t) spi_readRegister(SPI_REG_CONFIGB3);           //2p0 addition
    gDeviceSpiReg.configB4_reg = (uint16_t) spi_readRegister(SPI_REG_CONFIGB4);           //2p0 addition
    gDeviceSpiReg.cmd0_reg = (uint16_t) spi_readRegister(SPI_REG_CMD0);
    gDeviceSpiReg.cmd1_reg = (uint16_t) spi_readRegister(SPI_REG_CMD1);
    gDeviceSpiReg.cmd2_reg = (uint16_t) spi_readRegister(SPI_REG_CMD2);
} // end gui_regRead()

// This function handles synchronizing the device registers with the GUI
void gui_regUpdateTask(void)
{
    volatile uint8_t drvRegAddr;
    volatile uint16_t drvDataNew;

    // Write to single register using manual register write command
    if (gBrushedObj.ManWriteCmd)
    {
        drvRegAddr = gBrushedObj.ManWriteAddr;
        drvDataNew = gBrushedObj.ManWriteData;

        // This function will update the register global variable for the specific register that is manually written
        switch (drvRegAddr)
        {
        case SPI_REG_CONFIGA0:
            gDeviceSpiReg.configA0_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGA1:
            gDeviceSpiReg.configA1_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGA2:
            gDeviceSpiReg.configA2_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGA3:
            gDeviceSpiReg.configA3_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGA4:
            gDeviceSpiReg.configA4_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGA5:
            gDeviceSpiReg.configA5_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGA6:
            gDeviceSpiReg.configA6_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGB0:
            gDeviceSpiReg.configB0_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGB1:
            gDeviceSpiReg.configB1_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGB2:
            gDeviceSpiReg.configB2_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGB3:                                      //2p0 addition
            gDeviceSpiReg.configB3_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGB4:                                      //2p0 addition
            gDeviceSpiReg.configB4_reg = drvDataNew;
            break;
        case SPI_REG_CMD0:
            gDeviceSpiReg.cmd0_reg = drvDataNew;
            break;
        case SPI_REG_CMD1:
            gDeviceSpiReg.cmd1_reg = drvDataNew;
            break;
        case SPI_REG_CMD2:
            gDeviceSpiReg.cmd2_reg = drvDataNew;
            break;
        default:
            break;
        }

        // update upper and lower byte values of ConfigAx and ConfigBx registers from
        // their respective global variables for crc calculation
        configAx[0] = (uint8_t) (gDeviceSpiReg.configA0_reg >> SPI_UPPER_DATA_POS) & 0xFF; // CONFIGA0 upper byte
        configAx[1] = (uint8_t) (gDeviceSpiReg.configA0_reg) & 0xFF;                       // CONFIGA0 lower byte
        configAx[2] = (uint8_t) (gDeviceSpiReg.configA1_reg >> SPI_UPPER_DATA_POS) & 0xFF; // CONFIGA1 upper byte
        configAx[3] = (uint8_t) (gDeviceSpiReg.configA1_reg) & 0xFF;                       // CONFIGA1 lower byte
        configAx[4] = (uint8_t) (gDeviceSpiReg.configA2_reg >> SPI_UPPER_DATA_POS) & 0xFF; // CONFIGA2 upper byte
        configAx[5] = (uint8_t) (gDeviceSpiReg.configA2_reg) & 0xFF;                       // CONFIGA2 lower byte
        configAx[6] = (uint8_t) (gDeviceSpiReg.configA3_reg >> SPI_UPPER_DATA_POS) & 0xFF; // CONFIGA3 upper byte
        configAx[7] = (uint8_t) (gDeviceSpiReg.configA3_reg) & 0xFF;                       // CONFIGA3 lower byte
        configAx[8] = (uint8_t) (gDeviceSpiReg.configA4_reg >> SPI_UPPER_DATA_POS) & 0xFF; // CONFIGA4 upper byte
        configAx[9] = (uint8_t) (gDeviceSpiReg.configA4_reg) & 0xFF;                       // CONFIGA4 lower byte
        configAx[10] = (uint8_t) (gDeviceSpiReg.configA5_reg >> SPI_UPPER_DATA_POS) & 0xFF;// CONFIGA5 upper byte
        configAx[11] = (uint8_t) (gDeviceSpiReg.configA5_reg) & 0xFF;                      // CONFIGA5 lower byte
        configAx[12] = (uint8_t) (gDeviceSpiReg.configA6_reg >> SPI_UPPER_DATA_POS) & 0xFF;// CONFIGA6 upper byte
        configAx[13] = (uint8_t) (gDeviceSpiReg.configA6_reg) & 0xFF;                      // CONFIGA6 lower byte

        configBx[0] = (uint8_t) (gDeviceSpiReg.configB0_reg >> SPI_UPPER_DATA_POS) & 0xFF; // CONFIGB0 upper byte
        configBx[1] = (uint8_t) (gDeviceSpiReg.configB0_reg) & 0xFF;                       // CONFIGB0 lower byte
        configBx[2] = (uint8_t) (gDeviceSpiReg.configB1_reg >> SPI_UPPER_DATA_POS) & 0xFF; // CONFIGB1 upper byte
        configBx[3] = (uint8_t) (gDeviceSpiReg.configB1_reg) & 0xFF;                       // CONFIGB1 lower byte
        configBx[4] = (uint8_t) (gDeviceSpiReg.configB2_reg >> SPI_UPPER_DATA_POS) & 0xFF; // CONFIGB2 upper byte
        configBx[5] = (uint8_t) (gDeviceSpiReg.configB2_reg) & 0xFF;                       // CONFIGB2 lower byte
//additions for PG2p0
        configBx[6] = (uint8_t) (gDeviceSpiReg.configB3_reg >> SPI_UPPER_DATA_POS) & 0xFF; // CONFIGB3 upper byte
        configBx[7] = (uint8_t) (gDeviceSpiReg.configB3_reg) & 0xFF;                       // CONFIGB3 lower byte
        configBx[8] = (uint8_t) (gDeviceSpiReg.configB4_reg >> SPI_UPPER_DATA_POS) & 0xFF; // CONFIGB4 upper byte
        configBx[9] = (uint8_t) (gDeviceSpiReg.configB4_reg) & 0xFF;                       // CONFIGB4 lower byte

        spi_writeRegister((uint8_t) drvRegAddr, (uint16_t) drvDataNew);
        gBrushedObj.ManWriteCmd = false;
    }

    // Read all registers at once
    if (ReadCMD)
    {
        gui_regRead();
        ReadCMD = 0;
    }

    // Read single register using manual register read command
    if (gBrushedObj.ManReadCmd)
    {
        drvRegAddr = gBrushedObj.ManReadAddr;
        drvDataNew = spi_readRegister((uint8_t) drvRegAddr);

        gBrushedObj.ManReadData = drvDataNew;
        switch (drvRegAddr)
        {
        case SPI_REG_STATUS0:
            gDeviceSpiReg.status0_reg = drvDataNew;
            break;
        case SPI_REG_STATUS1:
            gDeviceSpiReg.status1_reg = drvDataNew;
            break;
        case SPI_REG_STATUS2:
            gDeviceSpiReg.status2_reg = drvDataNew;
            break;
        case SPI_REG_STATUS3:
            gDeviceSpiReg.status3_reg = drvDataNew;
            break;
        case SPI_REG_STATUS4:
            gDeviceSpiReg.status4_reg = drvDataNew;
            break;
        case SPI_REG_STATUS5:
            gDeviceSpiReg.status5_reg = drvDataNew;
            break;
        case SPI_REG_MEAS0:
            gDeviceSpiReg.meas0_reg = drvDataNew;
            break;
        case SPI_REG_MEAS1:
            gDeviceSpiReg.meas1_reg = drvDataNew;
            break;
        case SPI_REG_MEAS2:
            gDeviceSpiReg.meas2_reg = drvDataNew;
            break;
        case SPI_REG_MEAS3:
            gDeviceSpiReg.meas3_reg = drvDataNew;
            break;
        case SPI_REG_MEAS4:
            gDeviceSpiReg.meas4_reg = drvDataNew;
            break;
        case SPI_REG_MEAS5:
            gDeviceSpiReg.meas5_reg = drvDataNew;
            break;
        case SPI_REG_MEAS6:
            gDeviceSpiReg.meas6_reg = drvDataNew;
            break;

        case SPI_REG_CONFIGA0:
            gDeviceSpiReg.configA0_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGA1:
            gDeviceSpiReg.configA1_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGA2:
            gDeviceSpiReg.configA2_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGA3:
            gDeviceSpiReg.configA3_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGA4:
            gDeviceSpiReg.configA4_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGA5:
            gDeviceSpiReg.configA5_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGA6:
            gDeviceSpiReg.configA6_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGB0:
            gDeviceSpiReg.configB0_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGB1:
            gDeviceSpiReg.configB1_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGB2:
            gDeviceSpiReg.configB2_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGB3:
            gDeviceSpiReg.configB3_reg = drvDataNew;
            break;
        case SPI_REG_CONFIGB4:
            gDeviceSpiReg.configB4_reg = drvDataNew;
            break;
        case SPI_REG_CMD0:
            gDeviceSpiReg.cmd0_reg = drvDataNew;
            break;
        case SPI_REG_CMD1:
            gDeviceSpiReg.cmd1_reg = drvDataNew;
            break;
        case SPI_REG_CMD2:
            gDeviceSpiReg.cmd2_reg = drvDataNew;
            break;
        default:
            break;
        }
        gBrushedObj.ManReadCmd = false;
    }
} // end gui_regUpdateTask()

void driver_A_power_on(void)
{
    set_ENABLE1_B_lo;                                      // Disable ENABLE1 Driver B
    set_ENABLE2_B_lo;                                      // Disable ENABLE2 Driver B
    gBrushedObj.DRVB_enable1 = 0;                          // Global variable 0
    gBrushedObj.DRVB_enable2 = 0;                          // Global variable 0
    __delay_cycles(8000);                                  // allow delay for Driver B disable 0.5 ms
    set_SW_EN2_hi;                                         // Driver B power off
    P4OUT &= ~DRV_B_LED;                                   // Driver B LED off
    gBrushedObj.DRVB_sel = 0;                              // Update global variable
    __delay_cycles(8000);                                  // Power off settling delay 0.5 ms
    set_SW_EN1_lo;                                         // Driver A power 0n
    P4OUT |= DRV_A_LED;                                    // Driver A LED on
    __delay_cycles(8000);                                  // Power on settling delay 0.5ms
} // end driver_A_power_on()

void driver_B_power_on(void)
{
    set_ENABLE1_A_lo;                                      // Disable ENABLE1 Driver A
    set_ENABLE2_A_lo;                                      // Disable ENABLE2 Driver A
    gBrushedObj.DRVA_enable1 = 0;                          // Global variable 0
    gBrushedObj.DRVA_enable2 = 0;                          // Global variable 0
    __delay_cycles(8000);                                  // allow delay for Driver B disable 0.5 ms
    set_SW_EN1_hi;                                         // Driver A power off
    P4OUT &= ~DRV_A_LED;                                   // Driver A LED off
    gBrushedObj.DRVA_sel = 0;                              // Update global variable
    __delay_cycles(8000);                                  // Power off settling delay 0.5 ms
    set_SW_EN2_lo;                                         // Driver B power 0n
    P4OUT |= DRV_B_LED;                                    // Driver B LED on
    __delay_cycles(8000);                                  // Power on settling delay 0.5 ms
} // end driver_B_power_on()

/********************************************************************************
 * LOCAL ROUTINES
 *******************************************************************************/
void getEVMID(void)
{
    // Read the device ID from STATUS1 register
    /*gBrushedObj.deviceID = ((uint16_t) spi_readRegister(SPI_REG_STATUS1) >> SPI_DEVICE_ID_POS) & 0xF;

    if (gBrushedObj.deviceID == 1)
    {
        gBrushedObj.deviceID = 3946;                       // device is drv3946-q1
        gBrushedObj.revID = REV_1p0;                       // silicon revision 1p0
    }
    else if (gBrushedObj.deviceID == 2)
    {
        gBrushedObj.deviceID = 3946;                       // device is drv3946-q1
        gBrushedObj.revID = REV_2p0;                       // silicon revision 2p0
    }*/

    // Manual entry
    gBrushedObj.deviceID = 3946;                           // device is drv3946-q1
    gBrushedObj.revID = REV_1p0;                           // silicon revision 1p0
} // end getEVMID()

// MSP430G2955 Clock initialization
static inline void clockInit(void)
{
    // 16 MHz DCO for MCLK
    DCOCTL = 0x00;
    BCSCTL1= CALBC1_16MHZ;
    DCOCTL= CALDCO_16MHZ;

    // 2MHz SMCLK for PWM frequency and UART baud rate
    BCSCTL2 |= DIVS_3;
    BCSCTL3 |= LFXT1S_2;
} // end clockInit()

// MSP430G2955 GPIO initialization
static inline void gpioInit()
{
    // Port 1
    P1OUT = 0;
    P1DIR = 0;
    P1DIR |= (STATUS | TP13 | TP14 | ENABLE1_A | ENABLE2_A);
    P1SEL = 0;
    P1SEL2 = 0;

    // Port 2
    P2DIR = 0;
    P2DIR |= (FAULT_A_LED | FAULT_B_LED);
    P2SEL = 0;
    P2SEL2 = 0;
    ADC10AE0 |= (IPROPI2_A | IPROPI1_A | IPROPI1_B | IPROPI2_B | VBAT_MON); // analog input pins
    ADC10CTL0 |= ADC10ON;                                  // ADC10 ON
    ADC10CTL1 |= INCH_4;                                   // ADC channel A4 for measuring VBAT_MON voltage

    // Port 3
    P3DIR = 0;
    P3DIR |= (ENABLE1_B | ENABLE2_B);
    P3SEL2 = 0;

    // Port 4
    P4OUT = 0;
    P4DIR = 0;
    P4DIR |= (DRV_A_LED | GUI_STATUS | DRV_B_LED | TP17 | TP20 | TP21 | SW_EN1 | SW_EN2);
    P4SEL = 0;
    P4SEL2 = 0;

    // Setup PUSH button for triggering IRQ
    // External pull up resistor 47kOhms R18
    P1DIR &= ~PUSH;                                        // input mode
    P1IES |= PUSH;                                         // falling edge
    P1IFG &= ~PUSH;                                        // clear IRQ that may have been generated by setting IES
    // P1IE  |= PUSH;                                         // enable IRQ on PUSH button press **not enabled in this firmware version**
} // end gpioInit()

// Setup Watchdog Timer as an Interval Timer, Timer Interval to periodically wake up MSP430G2955 from LPM0
static inline void watchdogTimerInit(void)
{
    WDTCTL = WDT_ADLY_1_9;                                 // Configure time interval for periodic wake up and processing 1.9 ms if using 32 kHz ACLK, with VLO ~4 ms
    IE1 |= WDTIE;                                          // Watchdog interval timer Interrupt enable
} // end watchdogTimerInit()

void mcu_init(void)
{
    // Stop Watchdog Timer
    WDTCTL = WDTPW | WDTHOLD;

    // Setup CLKs, GPIO and Peripherals
    gpioInit();                                            // MSP430G2955 GPIOI initialization
    clockInit();                                           // MSP430G2955 basic clock initialization
    uartInit();                                            // MSP430G2955 UART initialization
    spiInit();                                             // MSP430G2955 SPI initialization
    watchdogTimerInit();
} // end initMcu()

void drv3946q1evm_init(void)
{
    gBrushedObj.firmwareVersion = FIRMWARE_VERSION;        // EVM Firmware version configuration
    gBrushedObj.drv_SPI_mode = 0;                          // 0 = peer to peer SPI, 1 = addressable SPI

    // Clear Drivers B and A faults on power on and configure Driver A for GUI start up
    driver_B_power_on();                                   // Driver B power on
    gBrushedObj.DRVB_sel = 1;                              // For driver B nSCS
    __delay_cycles(160000);                                // Power on settling delay 10ms
    drv_RESET();                                           // Driver clear fault to clear POR
    driver_A_power_on();                                   // Driver A power on
    gBrushedObj.DRVA_sel = 1;                              // For driver A nSCS and default device at GUI start up
    __delay_cycles(160000);                                // Power on settling delay 10ms

    while (ADC10MEM < 155)                                 // Check if VBAT > 4.5 V
    {
        __delay_cycles(6000000);                           // Delay 400ms
        P2OUT ^= (FAULT_A_LED | FAULT_B_LED);              // Toggle FAULT LEDs to indicate VBAT < 4.5 V
        ADC10CTL0 |= ENC + ADC10SC;                        // Enable and start conversion
        while (ADC10CTL1 & ADC10BUSY);                     // Loop until ADC conversion is completed
    }
    __delay_cycles(6000000);                               // Delay 400ms
    P2OUT &= (FAULT_A_LED | FAULT_B_LED);                  // FAULT LEDs off after VBAT > 4.5 V

    drv_RESET();                                           // Driver clear fault to clear POR flag
    RegReadPeriod = 0;                                     // Initialize Register read period
    ReadCMD = 0;                                           // Initialize Register read command
    gBrushedObj.AutoRead = 1;                              // Enable GUI Auto Read

    // Read device ID
    getEVMID();                                            // Get driver device ID
} // end drv3946q1evm_init()

inline void drv_RESET(void)
{
//    spi_writeRegister(SPI_REG_CMD2, 0xC000);               // write CLR_FAULT and RE_INIT_NAD bits high
//    __delay_cycles(20000);                                 // 1 ms settling delay
//    spi_writeRegister(SPI_REG_CMD2, 0x8000);               // write CLR_FAULT high and clear RE_INIT_NAD bit
//    __delay_cycles(100);                                   // 5 us settling delay
} // end drv_RESET()

// Set trigger for reading drv3946q1 Registers
void drv3946q1_RegistersRead()
{
    RegReadPeriod++;
    if (RegReadPeriod == REG_READ_PERIOD)
    {
        RegReadPeriod = 0;

        if (gBrushedObj.AutoRead)                          // Periodic read Registers and update Register memory buffer if Auto read in GUI is not off
        {
            ReadCMD = 1;
        }
    }
}// End Set trigger for reading drv3946q1 Registers
