/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  drv3946.h - Declaration file for utility functions and global variables
 *  DRV3946-Q1EVM_FIRMWARE
 *  10/14/2021
 *  Murugavel Raju
 *
 ******************************************************************************/

#ifndef DRV3946_H_
#define DRV3946_H_

/******************************************************************************
 * HEADER FILES
 *****************************************************************************/
#include "drv3946-q1/drv3946-q1-RegMap.h"
#include "msp430.h"

/******************************************************************************
 * MACROS
 *****************************************************************************/

// GPIO Port 1 Definitions
#define STATUS      BIT0           // P1.0: Status LED output
#define TP13        BIT1           // P1.1: Test Point TP13
#define TP14        BIT2           // P1.2: Test Point TP14
#define PUSH        BIT3           // P1.3: Push button
#define FAULT_A     BIT4           // P1.4: NAD/nFAULT for DRV A
#define FAULT_B     BIT5           // P1.5: NAD/nFAULT for DRV B
#define ENABLE1_A   BIT6           // P1.6: ENABLE1_A control
#define ENABLE2_A   BIT7           // P1.7: ENABLE2_A control

// GPIO Port 2 Definitions
#define IPROPI2_A   BIT0           // P2.0: IPROPI2_A analog input
#define IPROPI1_A   BIT1           // P2.1: IPROPI1_A analog input
#define IPROPI1_B   BIT2           // P2.2: IPROPI1_B analog input
#define IPROPI2_B   BIT3           // P2.3: IPROPI2_B analog input
#define VBAT_MON    BIT4           // P2.4: VBAT monitor analog input
#define FAULT_A_LED BIT5           // P2.5: FAULT_A status LED output
#define FAULT_B_LED BIT6           // P2.6: FAULT_B status LED output
#define nSCS_B      BIT7           // P2.7: nSCS_B control output

// GPIO Port 3 Definitions
#define nSCS_A      BIT0           // P3.0: STE nSCS_A control output UCB0STE
#define SDI         BIT1           // P3.1: Serial data input UCB0SIMO
#define SDO         BIT2           // P3.2: Serial data output UCB0SOMI
#define SCLK        BIT3           // P3.3: Serial clock output UCB0CLK
#define TX          BIT4           // P3.4: UART TXD UCA0TXD
#define RX          BIT5           // P3.5: UART RXD UCA0RXD
#define ENABLE1_B   BIT6           // P3.6: ENABLE1_B control
#define ENABLE2_B   BIT7           // P3.7: ENABLE2_B control

// GPIO Port 4 Definitions
#define DRV_A_LED   BIT0           // P4.0: DRV A status LED output
#define GUI_STATUS  BIT1           // P4.1: GUI status LED output
#define DRV_B_LED   BIT2           // P4.2: DRV B status LED output
#define TP17        BIT3           // P4.3: Test Point TP17
#define TP20        BIT4           // P4.4: Test Point TP20
#define TP21        BIT5           // P4.5: Test Point TP21
#define SW_EN1      BIT6           // P4.6: HS switch control signal for PVDD_A
#define SW_EN2      BIT7           // P4.7: HS switch control signal for PVDD_B

// gpio Macros
#define set_ENABLE1_A_hi       (P1OUT |= ENABLE1_A)   // Set ENABLE1_A high
#define set_ENABLE1_A_lo       (P1OUT &= ~ENABLE1_A)  // Set ENABLE1_A low
#define set_ENABLE2_A_hi       (P1OUT |= ENABLE2_A)   // Set ENABLE2_A high
#define set_ENABLE2_A_lo       (P1OUT &= ~ENABLE2_A)  // Set ENABLE2_A low
#define set_ENABLE1_B_hi       (P3OUT |= ENABLE1_B)   // Set ENABLE1_B high
#define set_ENABLE1_B_lo       (P3OUT &= ~ENABLE1_B)  // Set ENABLE1_B low
#define set_ENABLE2_B_hi       (P3OUT |= ENABLE2_B)   // Set ENABLE2_B high
#define set_ENABLE2_B_lo       (P3OUT &= ~ENABLE2_B)  // Set ENABLE2_B low
#define set_SW_EN1_hi          (P4OUT |= SW_EN1)      // Set SW_EN1 high
#define set_SW_EN1_lo          (P4OUT &= ~SW_EN1)     // Set SW_EN1 low
#define set_SW_EN2_hi          (P4OUT |= SW_EN2)      // Set SW_EN2 high
#define set_SW_EN2_lo          (P4OUT &= ~SW_EN2)     // Set SW_EN2 low

#define read_FAULT_A_pin       (P1IN & FAULT_A)       // Read FAULT_A status
#define read_FAULT_B_pin       (P1IN & FAULT_B)       // Read FAULT_B status
#define read_ENABLE1_A_pin     (P1IN & ENABLE1_A)     // Read ENABLE1_A status
#define read_ENABLE2_A_pin     (P1IN & ENABLE2_A)     // Read ENABLE2_A status
#define read_ENABLE1_B_pin     (P3IN & ENABLE1_B)     // Read ENABLE1_B status
#define read_ENABLE2_B_pin     (P3IN & ENABLE2_B)     // Read ENABLE2_B status
#define read_SW_EN1_pin        (P4IN & SW_EN1)        // Read SW_EN1 status
#define read_SW_EN2_pin        (P4IN & SW_EN2)        // Read SW_EN2 status

#define REV_1p0             1
#define REV_2p0             2

#define REG_READ_PERIOD    96                         // Register read period 96 x 4 ms = 384 ms

/******************************************************************************
* DATA STRUCTURES
 *****************************************************************************/
typedef struct DRV3946_Device_Obj
{
    // starting address = 0x0990, Number of bytes = 41 bytes
    float firmwareVersion;

    // Device Parameters
    volatile uint16_t deviceID;                       // device ID value
    volatile uint16_t revID;                          // device rev ID
    volatile uint8_t DRVA_enable1;                    // 0: OUT1A disabled , 1: OUT1A enabled
    volatile uint8_t DRVA_enable2;                    // 0: OUT2A disabled , 1: OUT2A enabled
    volatile uint8_t DRVB_enable1;                    // 0: OUT1B disabled , 1: OUT1B enabled
    volatile uint8_t DRVB_enable2;                    // 0: OUT2B disabled , 1: OUT2B enabled
    volatile uint8_t DRVA_FAULT;                      // 0: No Fault , 1: Fault
    volatile uint8_t DRVB_FAULT;                      // 0: No Fault , 1: Fault
    volatile uint8_t DRVA_sel;                        // DRVA selected
    volatile uint8_t DRVB_sel;                        // DRVB selected
    volatile uint8_t clrFlt;                          // clear button state 1: button pressed
    volatile uint8_t AutoRead;                        // 0: Auto Read off, 1: Auto Read on

    // IPROPI ADC reading *Not implemented in firmware*
    volatile uint16_t ipropiAdc1_avg;                 // average current IPROPI1
    volatile uint16_t ipropiAdc1_peak;                // peak current IPROPI1
    volatile uint16_t ipropiAdc2_avg;                 // average current IPROPI2
    volatile uint16_t ipropiAdc2_peak;                // peak current IPROPI2

    // Register Read Write
    uint16_t   ManWriteAddr;                          // Manual write Address variable
    uint16_t   ManReadAddr;                           // Manual read Address variable
    uint16_t   ManWriteData;                          // Manual write data variable
    uint16_t   ManReadData;                           // Manual read data variable   //32 bytes
    bool       WriteCmd;                              // Registers write GUI command
    bool       ManWriteCmd;                           // Manual Register write GUIcommand
    bool       ManReadCmd;                            // Manual Register read GUI command
    bool       ReadCmd;                               // Registers read GUI command

    // NAD global variable
    volatile uint8_t  NAD_A;                          // variable for DRV A NAD
    volatile uint8_t  NAD_B;                          // variable for DRV B NAD
    volatile uint8_t drv_SPI_mode;                    // SPI mode selection
} DRV3946_Q1_Device_Obj_t;

/******************************************************************************
 * VARIABLES FOR UTILITY
 *****************************************************************************/
volatile uint8_t RegReadPeriod;
volatile uint8_t ReadCMD;

/******************************************************************************
 * EXTERN DECLARATIONS
 *****************************************************************************/
extern volatile DRV3946_Q1_Device_Obj_t gBrushedObj;
extern volatile DRV3946_Q1_REG_t gDeviceSpiReg;
extern volatile uint8_t configAx[14];                 //Array for CONFIGAX upper and lower bytes
extern volatile uint8_t configBx[10];                  //Array for CONFIGBX upper and lower bytes

// Function Prototypes
void gui_deviceControlUpdateTask(void);
void gui_regUpdateTask(void);
void gui_regRead(void);
void driver_A_power_on(void);
void driver_B_power_on(void);
void mcu_init(void);
void getEVMID(void);
void drv3946q1evm_init(void);
inline void drv_RESET(void);
void drv3946q1_RegistersRead();

#endif // DRV3946_H_
