/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *  main.c - c file for main function
 *  DRV3946-Q1EVM_FIRMWARE
 *  10/14/2021
 *  Murugavel Raju
 ******************************************************************************/
#include <msp430.h>
#include "drv3946-q1/drv3946-q1.h"
#include "Serial_Cmd_Monitor.h"

// Globals
int timerUpdate = 0;                           // Timers are unused in this firmware version
unsigned int OFCount;                          // Variable for watchdog LED blink

void main(void)                                // Main function
{
    // Initialize MCU peripherals
    mcu_init();                                // Initialize and configure MSP430G2955

init:
    drv3946q1evm_init();                       // EVM hardware initialization

    // Enable interrupts
    __bis_SR_register(GIE);                    // Enable MSP430G2955 global interrupts

    // GUI Composer Monitor Initialization
    ClearBufferRelatedParam();

    while (1)                                  // Wake up and execute once in approximately 4 ms watchdog timer mode interrup using VLO
    {
        // P1OUT ^= TP14;                      // Toggle TP14 for probing with oscilloscope
        gui_deviceControlUpdateTask();         // GUI controls update with EVM hardware function
        gui_regUpdateTask();                   // DRV3946-Q1 Registers update function
        drv3946q1_RegistersRead();             // Set trigger for reading drv3946q1 Registers
        __bis_SR_register(LPM0_bits + GIE);    // Enter LPM0

        ADC10CTL0 |= ENC + ADC10SC;            // Enable and start conversion
        while (ADC10CTL1 & ADC10BUSY);         // Loop until ADC conversion is completed
        if (ADC10MEM < 155)                    // Check if VBAT > 4.5 V
            goto init;
    }
}// end main()

// Interrupt subroutines ISR
// Watchdog Interval Timer ISR
#pragma vector=WDT_VECTOR
__interrupt void WatchDog_Timer(void)
{
    ++OFCount;                                 // Status LED blink rate derived using the watchdog timer
    if (!(OFCount % 200))
        P1OUT ^= STATUS;                       // Blinking status LED indicates DRV3946Q1EVM firmware is executing correctly
    __bic_SR_register_on_exit(LPM0_bits);      // Clear LPM0 bits, wake up from LPM0 upon ISR exit
} // end WatchDog_Timer()

// TRAP: Trap Service routine for all unused ISRs
#pragma vector=TIMER0_A0_VECTOR, TIMER0_A1_VECTOR, TIMER1_A0_VECTOR, \
        TIMER1_A1_VECTOR, PORT2_VECTOR, COMPARATORA_VECTOR, NMI_VECTOR, \
        PORT1_VECTOR, ADC10_VECTOR, USCIAB0TX_VECTOR
__interrupt void Trap_ISR(void)
{
    // disable drivers and oscillate between STATUS and FAULT_A and FAULT_B
    // LEDs to indicate the firmware has crashed
    // Note: nFAULT is active low, and STATUS is active high
    P1OUT |= ENABLE1_A | ENABLE2_A;            // Disable DRV A
    P3OUT |= ENABLE1_B | ENABLE2_B;            // Disable DRV B
    P2OUT |= FAULT_A_LED | FAULT_B_LED;        // FAULT_A and FAULT_B LED on
    P1OUT &= ~STATUS;                          // Status LED off

    while (1)
    {
        P1OUT ^= STATUS;
        P2OUT ^= (FAULT_A_LED | FAULT_B_LED);
        _delay_cycles(10000000);
    }
}// end Trap_ISR()
