/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/******************************************************************************/
 #ifndef SPI_H_
 #define SPI_H_

// HEADER FILES
#include "msp430.h"
#include "stdint.h"

// SPI Protocol
#define SPI_BUSY_FLAG       0x01   // User define flag to check SPI busy status
#define SPI_ADDRESS_MASK    0x3E   // Mask for SPI register address bit
#define SPI_ADDRESS_POS     1      // Position for SPI register address bit
#define SPI_UPPER_DATA_MASK 0xFF00 // Mask for SPI register data bit
#define SPI_LOWER_DATA_MASK 0x00FF // Mask for SPI register data bit
#define SPI_DATA_MASK       0x00FF // Mask for SPI register data bit
#define SPI_DATA_POS        8      //
#define SPI_UPPER_DATA_POS  8      // Position for SPI register data bit
#define SPI_LOWER_DATA_POS  0      // Position for SPI register data bit
#define SPI_RW_BIT_MASK     0x01   // Mask for SPI register read write indication bit
#define SPI_NAD_MASK        0xC0   // Mask for NAD bits
#define SPI_NAD_POS         6      // Position for NAD bits
#define SPI_DEVICE_ID_POS   2      // Position for DEVICE_ID bits

// Function prototypes
void spiInit(void);

uint16_t spi_readRegister(uint8_t address);
/*
 * @brief     :Write the register value at given given address\n
 * @param[in] addr, Address of SPI resister
 * @return    uint16_t (0)
 */

uint16_t spi_writeRegister(uint8_t address, uint16_t data);
/*
 * @brief     :read the register value at given given address\n
 * @param[in] address, Address of SPI resister
 * @param[in] data, Value of register that need to write
 * @return    uint16_t (0)
 */

#endif // SPI_H_
